/*
$Id: main.cpp,v 1.21 2005/01/19 10:24:21 mah Exp $
$Log: main.cpp,v $
Revision 1.21  2005/01/19 10:24:21  mah
New command-line option --cvs-status to add `cvs status' information.
This information was always enabled before but this was too annoying if there
is no cvs information available.

Revision 1.20  2003/11/21 13:04:35  mah
unlimit stack size per default

Revision 1.19  2002/12/02 14:36:57  sbeyer
* ported source to gcc-3.2 (still compiles AND WORKS with gcc-2.9x)

Revision 1.18  2002/03/21 13:54:54  dirkl
*** empty log message ***

Revision 1.17  2001/08/21 11:08:06  dirkl
- removed error in simplification of negative constants

Revision 1.16  2001/07/28 11:15:56  cb
added --parse-only option

Revision 1.15  2001/06/22 13:02:57  dirkl
*** empty log message ***

Revision 1.14  2001/05/30 18:07:57  sbeyer
* added array and ram support in types
* BETA support for function lookup table

Revision 1.13  2001/05/17 12:12:19  dirkl
- added command-line option --no-translation

Revision 1.12  2001/02/05 11:14:45  sbeyer

added register support

Revision 1.11  2001/01/29 10:59:49  sbeyer

fixed command line parsing bug

Revision 1.10  2001/01/26 15:04:09  sbeyer

added includes for compatibility's sake (needed to compile under RedHat 7.0)

Revision 1.9  2001/01/25 15:06:52  sbeyer

added output file name support

Revision 1.8  2001/01/24 16:52:48  sbeyer

added full support for function bit selection (i.e. expr(i) = expr^i)

Revision 1.7  2001/01/21 17:22:58  sbeyer

added additional option '--hdl-comments'

Revision 1.6  2001/01/15 16:07:47  sbeyer

small bugfixes

Revision 1.5  2001/01/11 16:16:50  sbeyer

added command line support for pvs_path

Revision 1.4  2001/01/11 11:48:43  sbeyer

added parameter_parse_verbose support

*/
#include "config.h"
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include "synthesize.h"
#include "types_and_functions.h"
#include "tools.h"
#include "parser_typen.h"
// to unlimit stack:
#include <sys/resource.h>
#include <unistd.h>

using namespace std;

int main(int argc, char *argv[])
{
  int i=0;
  int erkannt=0;
  int special_path=0;
  char *std_path="/usr/local/pvs";

  pvs_path=std_path;
  parameter_gdlausgabe=0;
  parameter_parsetreeausgabe=0;
  parameter_verbose=0;
  parameter_parse_only=0;
  parameter_parse_verbose=0;
  parameter_speicherausgabe=0;
  parameter_hdl_comment=0;
  parameter_no_translation=0;
  parameter_cvs_status=0;
  freigegebener_speicher=0;
  allokierter_speicher=0;

  // unlimit stack size:
  {
	  struct rlimit limit;
	  getrlimit(RLIMIT_STACK, &limit);
	  limit.rlim_cur = limit.rlim_max;
	  setrlimit(RLIMIT_STACK, &limit);
  }

  // Parameter verarbeiten
  for (i=1;i<argc;i++)
    {
      if (strncmp(argv[i], "--", 2) != 0)
        {
	  break;  // this is a non-option
	}
      erkannt=0;
      if (strcmp(argv[i],"--pvs-path")==0) 
        {
          pvs_path=argv[++i];
          // remove trailing '/' (adding it would require more effort)
          if (pvs_path[strlen(pvs_path)-1]=='/')
            pvs_path[strlen(pvs_path)-1]=0;
          erkannt=1;
        }
      if (strcmp(argv[i],"--gdl")==0) 
        {
          parameter_gdlausgabe=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--parse-tree")==0)
        {
          parameter_parsetreeausgabe=1;
          erkannt=1;
        }        
      if (strcmp(argv[i],"--cvs-status")==0) 
        {
          parameter_cvs_status=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--hdl-comment")==0)
        {
          parameter_hdl_comment=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--no-translation")==0)
        {
          parameter_no_translation=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--verbose")==0)
        {
          parameter_verbose=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--parse-only")==0)
        {
          parameter_parse_only=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--parse-verbose")==0)
        {
          parameter_verbose=1;
          parameter_parse_verbose=1;
          erkannt=1;
        }
      if (strcmp(argv[i],"--help")==0 || strcmp(argv[i],"-h")==0)
        {
          print_help();
          erkannt=1;
          return(0);
        }
      if (strcmp(argv[i],"--memory")==0)
        {
          parameter_speicherausgabe=1;
          erkannt=1;
        }
      if (erkannt==0)
        {
            printf ("unknown option %s\n",argv[i]);
            print_help();
            exit(1);
        }
    }
  
  if (parameter_parse_only ? i!=argc-1 : i!=argc-3) // kein Dateiname fr die .pvs Datei/Outputdatei angegeben 
    {
      printf("wrong number of filenames given!\n\n");
      print_help();
      exit(1);
    }

  if (parameter_parse_only != 1 && strcmp(argv[argc-3],argv[argc-1])==0)
    {
      printf("outputfile matches inputfile!\n\n");
      print_help();
      exit(1);
    }

  // if (parameter_parsetreeausgabe)
  //    {
  //      parse_tree_ausgeben();
  //    }
   
  //  if (parameter_gdlausgabe)
  //    {
  //      gdl_ausgabe("parse_tree.gdl",wurzel);
  //    }

  // read config-file
  read_config_file();
  if (parameter_verbose)
    cout << predefined_function_count << " predefined functions read from config-file\n";

  if (parameter_parse_only == 1)
    {
      generate_type_and_function_lists(argv[argc-1]);
    }
  else
    {
      generate_type_and_function_lists(argv[argc-3]);    
      synthesize_root_functions(argv[argc-1],argv[argc-2]);
    }

  free_parse_trees();
  if (parameter_speicherausgabe)
    {
      printf ("Allokierter Speicher   : %d Bytes\n",allokierter_speicher);
      printf ("Freigegebener Speicher : %d Bytes\n",freigegebener_speicher);
    }

  return(0);
}
